Option Explicit

' ******* Script Constants ********

Const dfxLocaleEnglish = 0
Const dfxLocaleLocal = 1

Const dfxDelimiterComma = 0
Const dfxDelimiterTab = 1
Const dfxDelimiterSpace = 2
Const dfxDelimiterSemicolon = 3

' *********************************


' Beginning of Script


Dim dfxData
Dim dfxModel
Dim dfxParameter
Dim dfxDefinition
Dim dfxConstants 
Dim ParamName 
Dim ParamValue
Dim ParamError
Dim myRoots
Dim Root
Dim fso
Dim tf
Dim i


' Create the DataCollection object and import the tabbed delimited data
' file thurber.dat. The file contains 2 columns of named data labeled
' Density and Electron Mobility (the first non-blank line in the file
' contains the series names). The first column is the independent
' variable X (Density), the second column is the dependent
' variable Y (Electron Mobility). This will add 2 DataSeries objects to
' the DataCollection, one named "Density" the other named "Electron Mobility".
Set dfxData = CreateObject("DataFitXD2.DataCOllection")
dfxData.ImportNamedSeriesFromFile "thurber.dat", dfxDelimiterTab
    
' Set the RegressionKey properties for the DataCollection according to the
' above description. The following is equivalent to
' dfxData.Item("Density").RegressionKey = "X" since "Item" is the default property.
dfxData("Density").RegressionKey = "X"
dfxData("Electron Mobility").RegressionKey = "Y"

' The model being solved is:
' Y = (B1+B2*x+B3*x^2+B4*x^3) / (1+B5*x+B6*x^2+B7*x^3)
' where B1 through B7 are the fitting parameters, and X
' is the independent variable "Density".
' Create the model definition object so that we may define
' a multi-line user-defined model.
Set dfxDefinition = CreateObject("DataFitXD2.ModelDefinition")
With dfxDefinition
    .Description = "Semiconductor electron mobility modeling"
    .F1 = "(B1 + B2 * x + B3 * x ^ 2 + B4 * x ^ 3)"
    .F2 = "(1 + B5 * x + B6 * x ^ 2 + B7 * x ^ 3)"
    .Y = "F1/F2"
End With

' Create an instance of the NonlinearRegression object
Set dfxModel = CreateObject("DataFitXD2.NonlinearRegression")

With dfxModel
    
    ' Define the multi-line user-defined model
    ' according to the above definition in dfxDefinition.
    ' Since we are using a ModelDefinition object to define
    ' the model, use the DefineUserModelSub method.
    .DefineUserModelSub dfxDefinition
    
    ' Define the intial estimates for the model since this is a
    ' user-defined model. There are seven parameters, B1 through B7.
    ' Item is the default property of the RegressionParameters object,
    ' so it is not necessary.
    ' .RegressionParameters.Item("B1").InitialEstimate = 1000
    .RegressionParameters.Item("B1").InitialEstimate = 1000
    .RegressionParameters.Item("B2").InitialEstimate = 1000
    .RegressionParameters.Item("B3").InitialEstimate = 400
    .RegressionParameters.Item("B4").InitialEstimate = 40
    .RegressionParameters.Item("B5").InitialEstimate = 0.7
    .RegressionParameters.Item("B6").InitialEstimate = 0.3
    .RegressionParameters.Item("B7").InitialEstimate = 0.03
    
    ' Set the regression solution options
    .MaxIterations = 100
    .MaxUnchangedIterations = 10
    .RegressionTolerance = 0.0000000001
    
    ' Fit the model to the DataCollection dfxData
    .FitCollection dfxData


    ' If the model solved successfully, display some sample results. Otherwise,
    ' display the solution log information to diagnose why it may have failed.
    If .IsSolved Then
        
        ' Write the information out to a file
        Set fso = CreateObject("Scripting.FileSystemObject")
        Set tf = fso.CreateTextFile("results.dat", TRUE)
  
        ' Create an instance of the GlobalConstants object to 
        ' obtain the version information and write out the file header.
        Set dfxConstants = CreateObject("DataFitXD2.GlobalConstants")
        tf.WriteLine("DataFitX " & dfxConstants.dfxMajorVersion & "." & _
                      dfxConstants.dfxMinorVersion & "." & _
                      dfxConstants.dfxRevision & " Regression Results")
        tf.WriteLine dfxConstants.dfxCopyright
        Set dfxConstants = Nothing

        tf.WriteLine "Results for model: "
        tf.WriteLine "F1 = " & .ModelDefinition.F1
        tf.WriteLine "F2 = " & .ModelDefinition.F2
        tf.WriteLine "Y = " & .ModelDefinition.Y
        tf.WriteLine "Description: " & .ModelDefinition.Description

        ' Print information about the data
         tf.WriteLine "Data set: Thurber.dat"
         tf.WriteLine "Independent Variable (X): " & .ResultTables.Item(1).Name
         tf.WriteLine "Dependent Variable (Y): " & .ResultTables.Item(2).Name
         tf.WriteLine "Number of Observations = " & .NumObservations
         tf.WriteLine "XMin = " & dfxModel.ResultTables.Item(1).Minimum
         tf.WriteLine "XMax = " & dfxModel.ResultTables.Item(1).Maximum
         tf.WriteLine "YMin = " & dfxModel.ResultTables.Item(2).Minimum
         tf.WriteLine "YMax = " & dfxModel.ResultTables.Item(2).Maximum
         tf.WriteBlankLines(1)
        
        ' Print the parameter names, values and standard errors
         tf.WriteLine "Parameter Results:"
         For Each dfxParameter in .RegressionParameters
            ParamName = dfxParameter.Name
            ParamValue = dfxParameter.Value
            ParamError = dfxParameter.StandardError
            tf.WriteLine ParamName & " = " & ParamValue & ", Standard Error = " & ParamError
        Next
        tf.WriteBlankLines(1)

        ' Print goodness of fit information
         tf.WriteLine "Number of Nonlinear Iterations = " & .NumIterations
         tf.WriteLine "Residual Standard Error = " & .StandardError
         tf.WriteLine "R2 Value = " & .R2
         tf.WriteLine "Regression Sum of Squares = " & .SSR
         tf.WriteLine "Residual Sum of Squares = " & .SSE
         tf.WriteBlankLines(1)

         ' Other sample output
            
         ' Single Predicted value
         tf.WriteLine "Value of Y at X = 0.79: " & .PredictValue(0.79)

         ' Value of X at specified Y (Root)
         tf.WriteLine "Value(s) of X at Y = 1408.1046: "
         myRoots = .CalculateRootsAlt(1408.1046, -5, 2, 100)


         For i = 0 To UBound(myRoots)
           tf.WriteLine "Root at X = " & myRoots(i) 'line 154
         Next

        ' Derivative of Y at X = 2.1
        tf.WriteLine "Derivative at X = 2.1: " & .CalculateDerivative(2.1)
            
         ' Integration from X = -2.1 to 2.1
         tf.WriteLine "Integral from -2.1 to 2.1: " & .CalculateIntegral(-2.1, 2.1)          

        tf.close

       
        MsgBox "Solution successful - see results.dat"

    Else


        .SaveLog "log.txt"
        MsgBox "Solution failed - see log.txt"

    End If

End With



